@@ -10,12 +10,6 @@ if RUBY_PLATFORM == 'java'
   Jdbc::MySQL.load_driver
 end
 
-if ENV["TRAVIS"] == "true"
-  FIXTURE_COMMAND = "LOAD DATA INFILE"
-else
-  FIXTURE_COMMAND = "LOAD DATA LOCAL INFILE"
-end
-
 class Sphinx
   attr_accessor :host, :username, :password
 
@@ -45,14 +39,13 @@ class Sphinx
 
     structure = File.open('spec/fixtures/sql/structure.sql') { |f| f.read }
     structure.split(/;/).each { |sql| mysql_client.execute sql }
-    sql_file 'data.tsv' do |path|
-      mysql_client.execute <<-SQL
-        #{FIXTURE_COMMAND} '#{path}' INTO TABLE
-        `riddle`.`people` FIELDS TERMINATED BY ',' ENCLOSED BY "'" (gender,
-        first_name, middle_initial, last_name, street_address, city, state,
-        postcode, email, birthday)
-      SQL
-    end
+
+    mysql_client.execute <<-SQL
+      LOAD DATA LOCAL INFILE '#{fixtures_path}/sql/data.tsv' INTO TABLE
+      `riddle`.`people` FIELDS TERMINATED BY ',' ENCLOSED BY "'" (gender,
+      first_name, middle_initial, last_name, street_address, city, state,
+      postcode, email, birthday)
+    SQL
 
     mysql_client.close
   end
@@ -117,16 +110,4 @@ class Sphinx
   def running?
     pid && `ps #{pid} | wc -l`.to_i > 1
   end
-
-  def sql_file(name, &block)
-    file = Tempfile.new(name)
-    file.write File.read("#{fixtures_path}/sql/#{name}")
-    `chmod +r #{file.path}`
-    file.flush
-
-    block.call file.path
-
-    file.close
-    file.unlink
-  end
 end
